local L = LibStub("AceLocale-3.0"):GetLocale("ProximoGrid", true)
local media = LibStub("LibSharedMedia-3.0");
--Frame Options
ProximoGrid.options.args.Frame = {
	type = "group",
	name = L["Frame"],
	desc = L["Frame setup."],
	order = 120,
	args = {

		textLength = {
			type = "range",
			name = L["Center Text Length"],
			desc = L["Number of characters to show on Center Text Indicator."],
			get = function()
				return ProximoGrid.db.profile.unitFrame.textLength;
			end,
			set = function(v)
				ProximoGrid.db.profile.unitFrame.textLength = v;
				ProximoGrid.UnitFrame:SetTextLength(v);
			end,
			min = 2,
			max = 40,
			step = 1,
			order = 40,
		},
		unitFrameWidth = {
			type = "range",
			name = L["Unit Frame Width"],
			desc = L["Adjust the width of each unit's frame."],
			get = function()
				return ProximoGrid.db.profile.unitFrame.width;
			end,
			set = function(v)
				ProximoGrid.db.profile.unitFrame.width = v;
				ProximoGrid.UnitFrame:SetWidth(v);
			end,
			min = 10,
			max = 100,
			step = 1,
			order = 60,
		},
		unitFrameHeight = {
			type = "range",
			name = L["Unit Frame Height"],
			desc = L["Adjust the height of each unit's frame."],
			get = function()
				return ProximoGrid.db.profile.unitFrame.height;
			end,
			set = function(v)
				ProximoGrid.db.profile.unitFrame.height = v;
				ProximoGrid.UnitFrame:SetHeight(v);
			end,
			min = 10,
			max = 100,
			step = 1,
			order = 80,
		},
		texture = {
			type = "text",
			name = L["Frame Texture"],
			desc = L["Adjust the texture of each unit's frame."],
			order = 85,
			validate = media:List("statusbar"),
			get = function ()
				return ProximoGrid.db.profile.unitFrame.texture
			end,
			set = function (v)
				ProximoGrid.db.profile.unitFrame.texture = v
				local texture = media:Fetch("statusbar", v);
				ProximoGrid.UnitFrame:SetTexture(texture);
			end,
		},
		inverBarColor = {
			type = "toggle",
			name = L["Invert Bar Color"],
			desc = L["Switch the bar's color with it's background."],
			get = function()
				return ProximoGrid.db.profile.unitFrame.invertBarColor;
			end,
			set = function(v)
				ProximoGrid.db.profile.unitFrame.invertBarColor = v;
				ProximoGrid.UnitFrame:InvertBarColor();
			end,
			min = 10,
			max = 100,
			step = 1,
			order = 90,
		},
		cornerSize = {
			type = "range",
			name = L["Corner Size"],
			desc = L["Corner indicators size."],
			get = function()
				return ProximoGrid.db.profile.unitFrame.cornerSize;
			end,
			set = function(v)
				ProximoGrid.db.profile.unitFrame.cornerSize = v;
				ProximoGrid.UnitFrame:SetCornerSize(v);
			end,
			min = 1,
			max = 20,
			step = 1,
			order = 140,
		},
		font = {
			type = "text",
			name = L["Font"],
			desc = L["Adjust the font settings."],
			validate = media:List("font"),
			order = 150;
			get = function ()
				return ProximoGrid.db.profile.unitFrame.font
			end,
			set = function (v)
				ProximoGrid.db.profile.unitFrame.font = v
				local font = media:Fetch("font", v)
				local fontSize = ProximoGrid.db.profile.unitFrame.fontSize
				ProximoGrid.UnitFrame:SetFont(font, fontSize)
			end,
		},
		fontSize = {
			type = "range",
			name = L["Font Size"],
			desc = L["Font size."],
			get = function()
				return ProximoGrid.db.profile.unitFrame.fontSize;
			end,
			set = function(v)
				ProximoGrid.db.profile.unitFrame.fontSize = v;
				local font = media:Fetch("font", ProximoGrid.db.profile.unitFrame.font);
				ProximoGrid.UnitFrame:SetFont(font, v);
			end,
			min = 1,
			max = 20,
			step = 1,
			order = 160,
		},
		orientation = {
			type = "text",
			name = L["Orientation of Frame"],
			desc = L["Set frame orientation."],
			get = function ()
				return ProximoGrid.db.profile.unitFrame.frameOrientation
			end,
			set = function (v)
				ProximoGrid.db.profile.unitFrame.frameOrientation = v
				ProximoGrid.UnitFrame:SetFrameOrientation(v);
			end,
			validate={["VERTICAL"] = L["VERTICAL"], ["HORIZONTAL"] = L["HORIZONTAL"]},
			order = 180,
		},
		textOrientation = {
			type = "text",
			name = L["Orientation of Text"],
			desc = L["Set frame text orientation."],
			get = function ()
				return ProximoGrid.db.profile.unitFrame.textOrientation
			end,
			set = function (v)
				ProximoGrid.db.profile.unitFrame.textOrientation = v
				ProximoGrid.UnitFrame:SetTextOrientation(v);
			end,
			validate={["VERTICAL"] = L["VERTICAL"], ["HORIZONTAL"] = L["HORIZONTAL"]},
			order = 200,
		}
	}
}


--Creating the frame for each unit that is in the CCTargets table
function ProximoGrid.UnitFrame:CreatUnitFrames()
	for i=1,3*5 do
		local f = CreateFrame("Button", nil, ProximoGrid.Frame["MainFrame"], "SecureActionButtonTemplate");

		--f:Execute( [[ animation = 0]] )
		f:SetAttribute("_onupdate", 
			[[
				if (animation==nil) then
					animation=0;
				end
				if (UnitExists(self:GetAttribute("unit")) and animation<1) then
					animation = animation+0.25;
				elseif (not UnitExists(self:GetAttribute("unit")) and animation>0) then
					animation = animation-0.25;
				else
					control:SetAnimating(false);
				end
				if (animation>0) then
					self:Show();	
				else
					self:Hide();
				end
				self:SetAlpha(animation);
			]]
		);
		SecureHandlerWrapScript(f, "OnAttributeChanged", f,
			[[
				if (name == "state-unitexists") then
					if (value) then
						self:Show();
						--control:SetAnimating(true);
					else
						--self:Hide();
						--control:SetAnimating(true);
					end
				end
			]]
		);
		
		RegisterUnitWatch(f, true);
		f:SetFrameStrata("LOW");

		-- set media based on shared media
		

		f:RegisterForClicks("LeftButtonUp", "RightButtonUp", "MiddleButtonUp", "Button4Up", "Button5Up")

		-- set our left-click action
		--f:SetAttribute("type1", "macro")
		--f:SetAttribute("*type1", "macro")
		--if(i<6)then
		--	f:SetAttribute("unit", "target")
		--end
		-- tooltip support
		--f:SetScript("OnEnter", function(this) self:OnEnter(this) end)
		--f:SetScript("OnLeave", function(this) self:OnLeave(this) end)

		-- create bar BG (which users will think is the real bar, as it is the one that has a shiny color)
		-- this is necessary as there's no other way to implement status bars that grow in the other direction than normal
		f.BarBG = f:CreateTexture()
		f.BarBG:SetPoint("TOP", f, "TOP", 0, -1)
		f.BarBG:SetPoint("BOTTOM", f, "BOTTOM", 0, 1)
		f.BarBG:SetPoint("LEFT", f, "LEFT", 1, 0)
		f.BarBG:SetPoint("RIGHT", f, "RIGHT", -1, 0)

		-- create bar
		f.Bar = CreateFrame("StatusBar", nil, f)
		f.Bar:SetOrientation(ProximoGrid.db.profile.unitFrame.frameOrientation)
		f.Bar:SetMinMaxValues(0,100)
		f.Bar:SetValue(100)
		f.Bar:SetStatusBarColor(0,0,0,0.8)
		f.Bar:SetPoint("TOPLEFT", f.BarBG, "TOPLEFT")
		f.Bar:SetPoint("BOTTOMRIGHT", f.BarBG, "BOTTOMRIGHT")

		-- set texture
		f:SetNormalTexture(1,1,1,0)
		f:SetHighlightTexture("Interface\\QuestFrame\\UI-QuestTitleHighlight")
		f:SetAlpha(1);
		f:Show();
		for indicatorName, indicatorData in pairs(ProximoGrid.Indicators) do
			indicatorData.module:CreateIndicator(f);
		end
		
		ProximoGrid.UnitFrame:UpdateUnitFrame(f, i);
		ProximoGrid.UnitFrames[i] = f;
		
	end
	local texture = media:Fetch("statusbar", ProximoGrid.db.profile.unitFrame.texture);
	local tmpColor = ProximoGrid.db.profile.unitFrame.barColor
	ProximoGrid.UnitFrame:SetTexture(texture);
	ProximoGrid.UnitFrame:SetBarColor(tmpColor[1], tmpColor[2], tmpColor[3], tmpColor[4]);
	ProximoGrid.UnitFrame:SetWidth(ProximoGrid.db.profile.unitFrame.width);
	ProximoGrid.UnitFrame:SetHeight(ProximoGrid.db.profile.unitFrame.height);
	ProximoGrid.UnitFrame:UpdateUnitFrame(ProximoGrid.UnitFrames[3*5], 3*5);
end

function ProximoGrid.UnitFrame:UpdateUnitFrame(frame, frameNumber) 
	frame:ClearAllPoints();
	local groupAnchor = ProximoGrid.db.profile.frame.groupAnchor;
	local y = 1;
	local dirY = "BOTTOM";
	local dirInvY = "TOP";
	if (groupAnchor:find("TOP")~=nil) then
		y = -1;
		dirY = "TOP"
		dirInvY = "BOTTOM";
	end
	local x = -1;
	local dirX = "RIGHT";
	local dirInvX = "LEFT";
	if (groupAnchor:find("LEFT")~=nil) then
		x = 1;
		dirX = "LEFT";
		dirInvX = "RIGHT";
	end
	if (frameNumber == 1) then
		frame:SetPoint(groupAnchor, ProximoGrid.Frame["MainFrame"], groupAnchor, x*(ProximoGrid.db.profile.frame.padding+5), y*(ProximoGrid.db.profile.frame.padding+5));
	else
		
		if (ProximoGrid.db.profile.frame.horizontalGroups) then
			if ((frameNumber-1) % ProximoGrid:GetColumns() ~= 0) then
				frame:SetPoint(groupAnchor, ProximoGrid.UnitFrames[frameNumber-1], dirY..dirInvX, x*ProximoGrid.db.profile.frame.spacing, 0);	
			else
				frame:SetPoint(groupAnchor, ProximoGrid.UnitFrames[frameNumber-5], dirInvY..dirX, 0, y*ProximoGrid.db.profile.frame.spacing);
			end
		else
			if ((frameNumber-1) % ProximoGrid:GetColumns() ~= 0) then
				frame:SetPoint(groupAnchor, ProximoGrid.UnitFrames[frameNumber-1], dirInvY..dirX, 0, y*ProximoGrid.db.profile.frame.spacing);
			else
				frame:SetPoint(groupAnchor, ProximoGrid.UnitFrames[frameNumber-5], dirY..dirInvX, x*ProximoGrid.db.profile.frame.spacing, 0);
			end
		end
	end
end

function ProximoGrid.UnitFrame:GetIndicator(name)
	return 	ProximoGrid.Indicators[name]["module"];
end

function ProximoGrid.UnitFrame:SetFont(font, fontSize)
	ProximoGrid:EventDispatcher("PROXIMOGRID_TEXT_FONT_UPDATED", font, fontSize);
end


function ProximoGrid.UnitFrame:SetCornerSize(size)
	ProximoGrid:EventDispatcher("PROXIMOGRID_UNIT_FRAME_CORNER_SIZE_UPDATED", size);
end

function ProximoGrid.UnitFrame:SetTexture(texture)
	ProximoGrid.UnitFrame:WithAllFrames(function(uf)
		uf.Bar:SetStatusBarTexture(texture);
		uf.BarBG:SetTexture(texture);
	end);
	ProximoGrid:EventDispatcher("PROXIMOGRID_UNIT_FRAME_TEXTURE_UPDATED", texture);
end

function ProximoGrid.UnitFrame:SetWidth(width)
	if not InCombatLockdown() then
		ProximoGrid.UnitFrame:WithAllFrames(function(uf)
			uf:SetWidth(width)
			--uf.Bar:SetWidth(width-4)
			--uf.BarBG:SetWidth(width-4)
		end);
		ProximoGrid:UpdateFrameSize();
		ProximoGrid:EventDispatcher("PROXIMOGRID_UNIT_FRAME_WIDTH_UPDATED", width);
	end
end

function ProximoGrid.UnitFrame:SetTextLength(length)
	ProximoGrid:EventDispatcher("PROXIMOGRID_TEXT_LENGTH_UPDATED", length);
end

function ProximoGrid.UnitFrame:SetHeight(height)
	if not InCombatLockdown() then
		ProximoGrid.UnitFrame:WithAllFrames(function(uf)
			uf:SetHeight(height)
			uf.Bar:SetHeight(height-4)
			uf.BarBG:SetHeight(height-4)
		end);
		ProximoGrid:UpdateFrameSize();
		ProximoGrid:EventDispatcher("PROXIMOGRID_UNIT_FRAME_HEIGHT_UPDATED", HEIGHT);
	end
end

function ProximoGrid.UnitFrame:WithAllFrames(func)
	for number,uf in pairs(ProximoGrid.UnitFrames) do
		func(uf);
	end
end

function ProximoGrid.UnitFrame:SetFrameOrientation(orientation)
	ProximoGrid.UnitFrame:WithAllFrames(function(uf)
		uf.Bar:SetOrientation(orientation);
	end)
	ProximoGrid:EventDispatcher("PROXIMOGRID_FRAME_ORIENTATION_UPDATED", orientation);
end



function ProximoGrid.UnitFrame:SetTextOrientation(orientation)
	ProximoGrid:EventDispatcher("PROXIMOGRID_TEXT_ORIENTATION_UPDATED", orientation);
end

function ProximoGrid.UnitFrame:SetBarColor(r, g, b, a, frame)
	local func = function(uf)
		if (ProximoGrid.db.profile.unitFrame.invertBarColor) then
			uf.Bar:SetStatusBarColor(r, g, b, a)
			uf.BarBG:SetVertexColor(0, 0, 0, 0)
		else
			uf.Bar:SetStatusBarColor(0, 0, 0, 0.8)
			uf.BarBG:SetVertexColor(r, g, b ,a)
		end
	end
	if (frame==nil) then
		ProximoGrid.UnitFrame:WithAllFrames(func);
	else
		func(frame);
	end
end

function ProximoGrid.UnitFrame:InvertBarColor()
	ProximoGrid.UnitFrame:WithAllFrames(function(uf)
		local r, g, b, a
		if (ProximoGrid.db.profile.unitFrame.invertBarColor) then
			r, g, b, a = uf.BarBG:GetVertexColor()
		else
			r, g, b, a = uf.Bar:GetStatusBarColor()
		end
		ProximoGrid.UnitFrame:SetBarColor(r, g, b, a, uf);
	end);
	ProximoGrid:EventDispatcher("PROXIMOGRID_UNIT_FRAME_INVERT_BAR_COLOR", ProximoGrid.db.profile.unitFrame.invertBarColor);
end