local L = LibStub("AceLocale-3.0"):GetLocale("ProximoGrid", true)
local defaultDB = {
	castingSpell = {
		color = {1, 0, 0, 1},
		priority = 85
	},
	spellGroups = {
		["Heals"] = {
			color = {0, 0, 1, 1},
			priority = 85,		
			spells = {
				[select(1,GetSpellInfo(2061))] = true, --Flash Heal
				[select(1,GetSpellInfo(2060))] = true, --Greater Heal
				[select(1,GetSpellInfo(331))] = true, --Healing Wave
				[select(1,GetSpellInfo(1064))] = true, --Chain Heal
				[select(1,GetSpellInfo(2050))] = true, --Lesser Heal
				[select(1,GetSpellInfo(2054))] = true, --Heal
				[select(1,GetSpellInfo(596))] = true, --Prayer of Healing
				[select(1,GetSpellInfo(54518))] = true, --Penance
				[select(1,GetSpellInfo(19750))] = true, --Flash of Light
				[select(1,GetSpellInfo(635))] = true, --Holy Light
				[select(1,GetSpellInfo(8004))] = true, --Lesser Healing Wave
				[select(1,GetSpellInfo(5185))] = true, --Healing Touch
				[select(1,GetSpellInfo(8936))] = true, --Regrowth
			}
		},
		["Mana Burn"] = {
			color = {0, 0, 1, 1},
			priority = 85,
			spells = {
				[select(1,GetSpellInfo(8129))] = true, --Mana Burn
				[select(1,GetSpellInfo(5138))] = true, --Drain Mana
			}
		}
	}
};
StatusCastingSpell = ProximoGridStatus.NewModule("StatusCastingSpell", defaultDB);

StatusCastingSpell.lastUpdate = 0;
StatusCastingSpell.updateFrame = CreateFrame("frame");
StatusCastingSpell.cache = {};
StatusCastingSpell.header = L["Spell Casting"];
StatusCastingSpell.options = {
	anySpellCast = {
		type = "group",
		name = L["Any Spell Cast"],
		desc = L["Options for the status."],
		args = {
			color = {
				type = "color",
				name = L["Color"],
				desc = L["Color."],
				get = function()
					local tmpColor = StatusCastingSpell.db.castingSpell.color;
					return tmpColor[1], tmpColor[2], tmpColor[3], tmpColor[4]
				end,
				set = function(r, g, b, a)
					local tmpColor = StatusCastingSpell.db.castingSpell.color;
					tmpColor[1], tmpColor[2], tmpColor[3], tmpColor[4] = r, g, b, a or 1
				end,
				hasAlpha = false,
			},
			alpha = {
				type = "range",
				name = L["Alpha"],
				desc = L["Alpha."],
				get = function()
					return StatusCastingSpell.db.castingSpell.color[4];
				end,
				set = function(v)
					StatusCastingSpell.db.castingSpell.color[4] = v;
				end,
				min = 0,
				max = 1,
				step = 0.01
			},
			priority = {
				type = "range",
				name = L["Priority"],
				desc = L["Priority."],
				get = function()
					return StatusCastingSpell.db.castingSpell.priority;
				end,
				set = function(v)
					StatusCastingSpell.db.castingSpell.priority = v;
				end,
				min = 1,
				max = 99,
				step = 1
			}
		}
	},
	spellGroups = {
		type = "group",
		name = L["Spell Group"],
		desc = L["Spell group."],
		args = {
			addGroup = {
				type = "group",
				name = L["Add Group"],
				desc = L["Add a new group."],
				order = 100,
				args = {
					addSpellGroup = {
						type = "text",
						name = L["Add Buff Group"],
						desc = L["Add a new buff group"],
						get = false,
						usage = L["<spell name>"],
						set = function(value)
							if (StatusCastingSpell.db.spellGroups[value] ~= nil and StatusCastingSpell.db.spellGroups[value].removed == true) then
								StatusCastingSpell.db.spellGroups[value].removed = nil;
								StatusCastingSpell:SetupOptions();
							end
							if (value=="" or value == " " or StatusCastingSpell.db.spellGroups[value] ~= nil) then
								return;
							end
							StatusCastingSpell.db.spellGroups[value] = {
								color = {1, 0, 0, 1},
								priority = 90,
								spells = {}
							}
							StatusCastingSpell:SetupOptions();
						end
					}
				},
			},
			removeGroup = {
				type = "group",
				name = L["Remove Group"],
				desc = L["Remove a group you have added."],
				order = 100,
				args = {},
			},
		}
	},
}

--Only used in BGs and checks for spell casts
function StatusCastingSpell:COMBAT_LOG_EVENT_UNFILTERED(arg1, combatevent, sourceGUID, sourceName, sourceFlags, destGUID, destName, destFlags, spellId, spellName, spellSchool, missType, extraSpellName, extraSchool, auraType)
	-- Checks if someone that is not player controled is casting a spell
	if (combatevent == "SPELL_CAST_START" and bit.band(COMBATLOG_OBJECT_CONTROL_PLAYER, sourceFlags) ~= 0 and bit.band(COMBATLOG_OBJECT_AFFILIATION_OUTSIDER, sourceFlags) ~= 0) then
		local unitID = sourceGUID;
		if (ProximoGrid:GetUnitFrameByUnitID(unitID)~=nil) then
			local spellIcon, spellCastTime;
			spellIcon = select(3, GetSpellInfo(spellId));
			spellCastTime = select(7, GetSpellInfo(spellId));
			for groupName, spellGroup in pairs (StatusCastingSpell.db.spellGroups) do
				local spellFound = false;
				if (spellGroup.removed == nil or not spellGroup.removed) then
					for spellInGroupName, enabled in pairs(spellGroup.spells) do
						if (spellName == spellInGroupName) then
							spellFound = true;
							break;
						end
					end
					--if the casted spell matches a spell in the spell group
					if (spellFound) then
						local r,g,b,a = spellGroup.color[1], spellGroup.color[2], spellGroup.color[3], spellGroup.color[4];
						if (StatusCastingSpell.cache[unitID] == nil) then
							StatusCastingSpell.cache[unitID] = {};
						end
						StatusCastingSpell.cache[unitID][groupName] = GetTime()+spellCastTime/1000;
						if (StatusCastingSpell:SendStatusGained(unitID, sourceName, "statusCastingSpell"..groupName, spellGroup.priority, true, {["r"] = r,["g"] = g,["b"] = b, ["a"] = a}, spellName,  0, 0, spellIcon, 0, 0, 0)) then
							StatusAuras:SendAnnouncement(string.format(L["%s begins casting %s"], sourceName, spellName), "announcementCastingSpell"..groupName);
						end
					else
						StatusCastingSpell:SendStatusLost(unitID, "statusCastingSpell"..groupName);
					end
				end
			end
			if (StatusCastingSpell.cache[unitID] == nil) then
				StatusCastingSpell.cache[unitID] = {};
			end
			StatusCastingSpell.cache[unitID]["AnySpell"] = GetTime()+spellCastTime/1000;
			local r,g,b,a = StatusCastingSpell.db.castingSpell.color[1], StatusCastingSpell.db.castingSpell.color[2], StatusCastingSpell.db.castingSpell.color[3], StatusCastingSpell.db.castingSpell.color[4];
			StatusCastingSpell:SendStatusGained(unitID, sourceName, "statusCastingSpellAnySpell", StatusCastingSpell.db.castingSpell.priority, true, {["r"] = r,["g"] = g,["b"] = b, ["a"] = a}, spellName,  0, 0, spellIcon, 0, 0, 0);
		end
	end
end

--Only used in arenas
function StatusCastingSpell:PROXIMOGRID_UNIT_SPELL_CASTING_START(unitID, spellName, channeled)
	local unitName = ProximoGrid.Opponents[unitID].unitName;
	local spell, rank, displayName, icon, startTime, endTime, isTradeSkill;
	if (channeled) then
		spell, rank, displayName, icon, startTime, endTime, isTradeSkill = UnitChannelInfo(unitID)
	else
		spell, rank, displayName, icon, startTime, endTime, isTradeSkill = UnitCastingInfo(unitID)
	end

	for groupName, spellGroup in pairs (StatusCastingSpell.db.spellGroups) do
		if (spellGroup.removed == nil or not spellGroup.removed) then
			local spellFound = false;
			for spellInGroupName, enabled in pairs(spellGroup.spells) do
				if (spellName == spellInGroupName) then
					spellFound = true;
					break;
				end
			end
			if (spellFound) then
				local r,g,b,a = spellGroup.color[1], spellGroup.color[2], spellGroup.color[3], spellGroup.color[4];
				if (StatusCastingSpell:SendStatusGained(unitID, unitName, "statusCastingSpell"..groupName, spellGroup.priority, true, {["r"] = r,["g"] = g,["b"] = b, ["a"] = a}, spell,  0, 0, icon, startTime/1000, endTime/1000-GetTime(), 0)) then
					StatusAuras:SendAnnouncement(string.format(L["%s begins casting %s"], unitName, spell), "announcementCastingSpell"..groupName);
				end
			else
				StatusCastingSpell:SendStatusLost(unitID, "statusCastingSpell"..groupName);
			end
		end
	end
	local r,g,b,a = StatusCastingSpell.db.castingSpell.color[1], StatusCastingSpell.db.castingSpell.color[2], StatusCastingSpell.db.castingSpell.color[3], StatusCastingSpell.db.castingSpell.color[4];
	StatusCastingSpell:SendStatusGained(unitID, unitName, "statusCastingSpellAnySpell", StatusCastingSpell.db.castingSpell.priority, true, {["r"] = r,["g"] = g,["b"] = b, ["a"] = a}, spell,  0, 0, icon, startTime/1000, endTime/1000-GetTime(), 0);
end

--Only used in arenas
function StatusCastingSpell:PROXIMOGRID_UNIT_SPELL_CASTING_UPDATE(unitID, spell, channeled)
	local unitName = ProximoGrid.Opponents[unitID].unitName;
	local spell, rank, displayName, icon, startTime, endTime, isTradeSkill;
	if (channeled) then
		spell, rank, displayName, icon, startTime, endTime, isTradeSkill = UnitChannelInfo(unitID)
	else
		spell, rank, displayName, icon, startTime, endTime, isTradeSkill = UnitCastingInfo(unitID)
	end

	for groupName, spellGroup in pairs (StatusCastingSpell.db.spellGroups) do
		local spellFound = false;
		if (spellGroup.removed == nil or not spellGroup.removed) then
			for spellInGroupName, enabled in pairs(spellGroup.spells) do
				if (spellName == spellInGroupName) then
					spellFound = true;
					break;
				end
			end
			if (spellFound) then
				local r,g,b = spellGroup.color[1], spellGroup.color[2], spellGroup.color[3], spellGroup.color[4];
				StatusCastingSpell:SendStatusGained(unitID, unitName, "statusCastingSpell"..groupName, spellGroup.priority, true, {["r"] = r,["g"] = g,["b"] = b, ["a"] = a}, spell,  0, 0, icon, startTime/1000, endTime/1000-GetTime(), 0)
			else
				StatusCastingSpell:SendStatusLost(unitID, "statusCastingSpell"..groupName);
			end
		end
	end
	local r,g,b = StatusCastingSpell.db.castingSpell.color[1], StatusCastingSpell.db.castingSpell.color[2], StatusCastingSpell.db.castingSpell.color[3], StatusCastingSpell.db.castingSpell.color[4];
	StatusCastingSpell:SendStatusGained(unitID, unitName, "statusCastingSpellAnySpell", StatusCastingSpell.db.castingSpell.priority, true, {["r"] = r,["g"] = g,["b"] = b, ["a"] = a}, spell,  0, 0, icon, startTime/1000, endTime/1000-GetTime(), 0);
end

--Only used in arenas
function StatusCastingSpell:PROXIMOGRID_UNIT_SPELL_CASTING_END(unitID, spell, channeled)
	for groupName, spellGroup in pairs (StatusCastingSpell.db.spellGroups) do
		if (spellGroup.removed == nil or not spellGroup.removed) then
			StatusCastingSpell:SendStatusLost(unitID, "statusCastingSpell"..groupName);
		end
	end
	StatusCastingSpell:SendStatusLost(unitID, "statusCastingSpellAnySpell");
end

function StatusCastingSpell:OnInitialize()
	StatusCastingSpell:RegisterStatus("statusCastingSpellAnySpell", L["Any Spell"]);
	StatusCastingSpell:SetupOptions();
end

function StatusCastingSpell:OnEnable()
	if (ProximoGrid:inArenaZone()) then
		StatusCastingSpell:RegisterEvent("PROXIMOGRID_UNIT_SPELL_CASTING_START");
		StatusCastingSpell:RegisterEvent("PROXIMOGRID_UNIT_SPELL_CASTING_END");
		StatusCastingSpell:RegisterEvent("PROXIMOGRID_UNIT_SPELL_CASTING_UPDATE");
	else
		StatusCastingSpell:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED");
	end
	StatusCastingSpell.updateFrame:SetScript("OnUpdate", StatusCastingSpell.OnUpdate);
end

function StatusCastingSpell:OnDisable()
	if (self:IsEventRegistered("COMBAT_LOG_EVENT_UNFILTERED")) then
		StatusCastingSpell:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED");
		StatusCastingSpell.frame:SetScript("OnUpdate", nil);
	else
		StatusCastingSpell:UnregisterEvent("PROXIMOGRID_UNIT_SPELL_CASTING_START");
		StatusCastingSpell:UnregisterEvent("PROXIMOGRID_UNIT_SPELL_CASTING_END");
		StatusCastingSpell:UnregisterEvent("PROXIMOGRID_UNIT_SPELL_CASTING_UPDATE");
	end
end

function StatusCastingSpell:SetupOptions()
	local groupOption = StatusCastingSpell.options.spellGroups.args;
	for groupName, spellGroup in pairs (StatusCastingSpell.db.spellGroups) do
		if (spellGroup.removed == nil) then
			StatusCastingSpell:RegisterStatus("statusCastingSpell"..groupName, string.format(L["Spell Group: %s"], groupName));
			StatusCastingSpell:RegisterAnnouncement("announcementCastingSpell"..groupName, string.format(L["Spell Group: %s"], groupName));
			groupOption[groupName] = {
				type = "group",
				name = groupName,
				desc = string.format("Options for %s.", groupName),
				order = 10,
				args = {
					color = {
						type = "color",
						name = L["Color"],
						desc = L["Color."],
						get = function()
							local tmpColor = spellGroup.color;
							return tmpColor[1], tmpColor[2], tmpColor[3], tmpColor[4]
						end,
						set = function(r, g, b, a)
							local tmpColor = spellGroup.color;
							tmpColor[1], tmpColor[2], tmpColor[3], tmpColor[4] = r, g, b, a or 1
						end,
						hasAlpha = false,
					},
					alpha = {
						type = "range",
						name = L["Alpha"],
						desc = L["Alpha."],
						get = function()
							return spellGroup.color[4];
						end,
						set = function(v)
							spellGroup.color[4] = v;
						end,
						min = 0,
						max = 1,
						step = 0.01
					},
					priority = {
						type = "range",
						name = L["Priority"],
						desc = L["Priority."],
						get = function()
							return spellGroup.priority;
						end,
						set = function(v)
							spellGroup.priority = v;
						end,
						min = 1,
						max = 99,
						step = 1
					},
					spells = {
						type = "group",
						name = L["Spells"],
						desc = L["Spells in this spell group"],
						args = {},
						order = 40
					},

					addSpell = {
						type = "group",
						name = L["Add Spell"],
						desc = L["Add a new spell to this group."],
						order = 50,
						args = {
							name = {
								type = "text",
								name = L["Add Spell"],
								desc = L["Add a new spell to this group."],
								get = false,
								usage = L["<spell name>"],
								set = function(value)
									if (value=="" or value == " " or spellGroup.spells[value] ~= nil) then
										return;
									end
									spellGroup.spells[value] = true;
									groupOption[groupName].args.spells.args[value] = {
										type = "toggle",
										name = value,
										desc = string.format("Toggle %s", value);
										get = function()
											return spellGroup.spells[value];
										end,
										set = function(v)
											spellGroup.spells[value] = v;
										end
									}
									groupOption[groupName].args.removeSpell.args[value] = {
										type = "execute",
										name = string.format(L["Remove %s"], value),
										desc = string.format(L["Remove %s from this group."], value),
										func = function()
											groupOption[groupName].args.spells.args[value] = nil;
											spellGroup.spells[value] = nil;
											groupOption[groupName].args.removeSpell.args[value] = nil;
										end,
										order = 3
									}
								end
							}
						}
					},
					removeSpell = {
						type = "group",
						name = L["Remove spell"],
						desc = L["Remove a spell from this group."],
						order = 60,
						args = {}
					}
				}

			}
			for spellName, enable in pairs(spellGroup.spells) do
				groupOption[groupName].args.spells.args[spellName] = {
					type = "toggle",
					name = spellName,
					desc = string.format("Toggle %s", spellName);
					get = function()
						return spellGroup.spells[spellName];
					end,
					set = function(v)
						spellGroup.spells[spellName] = v;
					end
				}
				if (defaultDB.spellGroups[groupName] == nil or defaultDB.spellGroups[groupName].spells[spellName] == nil) then
					groupOption[groupName].args.removeSpell.args[spellName] = {
						type = "execute",
						name = string.format(L["Remove %s"], spellName),
						desc = string.format(L["Remove %s from this group."], spellName),
						func = function()
							groupOption[groupName].args.spells.args[spellName] = nil;
							spellGroup.spells[spellName] = nil;
							groupOption[groupName].args.removeSpell.args[spellName] = nil;
						end,
						order = 3
					}
				end
			end
			groupOption.removeGroup.args[groupName] = {
				type = "execute",
				name = string.format(L["Remove %s"], groupName),
				desc = string.format(L["Remove %s."], groupName),
				func = function()
					StatusCastingSpell:UnregisterStatus("statusCastingSpell"..groupName);
					StatusCastingSpell:UnregisterAnnouncement("announcementCastingSpell"..groupName);
					groupOption.removeGroup.args[groupName] = nil;
					if (defaultDB.spellGroups[groupName] == nil) then
						StatusCastingSpell.db.spellGroups[groupName] = nil;
					else
						StatusCastingSpell.db.spellGroups[groupName].removed = true;
					end
					groupOption[groupName] = nil;
				end,
				order = 3
			}
		end
	end
end

function StatusCastingSpell:OnUpdate(self, elapsed)
	if (StatusCastingSpell.lastUpdate+0.1<GetTime()) then
		for unitID, groups in pairs(StatusCastingSpell.cache) do
			local counter = 0;
			local removed = false;
			for groupName, time in pairs(groups) do
				counter = counter + 1; 
				if (time < GetTime()) then
					StatusCastingSpell.cache[unitID][groupName] = nil;
					StatusCastingSpell:SendStatusLost(unitID, "statusCastingSpell"..groupName);
					removed = true;
				end
			end
			if (counter == 1 and removed) then
				StatusCastingSpell.cache[unitID] = nil;
			end
		end
		StatusCastingSpell.lastUpdate = GetTime();
	end
end